@@ -1,9 +1,7 @@
-# coding=utf-8
-#
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2019 David R. MacIver
+# Most of this work is copyright (C) 2013-2020 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -15,22 +13,19 @@
 #
 # END HEADER
 
-from __future__ import absolute_import, division, print_function
-
 import collections
 import decimal
 import fractions
 import math
 from datetime import date, datetime, time, timedelta
+from ipaddress import IPv4Network, IPv6Network
 
 import pytest
 
-import hypothesis.strategies as ds
-from hypothesis import given, settings
+from hypothesis import given, settings, strategies as ds
 from hypothesis.errors import InvalidArgument
 from hypothesis.vendor.pretty import pretty
 from tests.common.debug import minimal
-from tests.common.utils import checks_deprecated_behaviour
 
 # Use `pretty` instead of `repr` for building test names, so that set and dict
 # parameters print consistently across multiple worker processes with different
@@ -59,21 +54,26 @@ def fn_ktest(*fnkwargs):
 
 
 @fn_ktest(
-    (ds.integers, {"min_value": float("nan")}),
+    (ds.integers, {"min_value": math.nan}),
     (ds.integers, {"min_value": 2, "max_value": 1}),
-    (ds.integers, {"min_value": float("nan")}),
-    (ds.integers, {"max_value": float("nan")}),
+    (ds.integers, {"min_value": math.nan}),
+    (ds.integers, {"max_value": math.nan}),
+    (ds.integers, {"min_value": decimal.Decimal("1.5")}),
+    (ds.integers, {"max_value": decimal.Decimal("1.5")}),
+    (ds.integers, {"min_value": -1.5, "max_value": -0.5}),
+    (ds.integers, {"min_value": 0.1, "max_value": 0.2}),
     (ds.dates, {"min_value": "fish"}),
     (ds.dates, {"max_value": "fish"}),
     (ds.dates, {"min_value": date(2017, 8, 22), "max_value": date(2017, 8, 21)}),
     (ds.datetimes, {"min_value": "fish"}),
     (ds.datetimes, {"max_value": "fish"}),
+    (ds.datetimes, {"allow_imaginary": 0}),
     (
         ds.datetimes,
         {"min_value": datetime(2017, 8, 22), "max_value": datetime(2017, 8, 21)},
     ),
-    (ds.decimals, {"min_value": float("nan")}),
-    (ds.decimals, {"max_value": float("nan")}),
+    (ds.decimals, {"min_value": math.nan}),
+    (ds.decimals, {"max_value": math.nan}),
     (ds.decimals, {"min_value": 2, "max_value": 1}),
     (ds.decimals, {"max_value": "-snan"}),
     (ds.decimals, {"max_value": complex(1, 2)}),
@@ -92,25 +92,30 @@ def fn_ktest(*fnkwargs):
         ds.dictionaries,
         {"keys": ds.booleans(), "values": ds.booleans(), "min_size": 10, "max_size": 1},
     ),
-    (ds.floats, {"min_value": float("nan")}),
-    (ds.floats, {"max_value": float("nan")}),
+    (ds.floats, {"min_value": math.nan}),
+    (ds.floats, {"max_value": math.nan}),
     (ds.floats, {"min_value": complex(1, 2)}),
     (ds.floats, {"max_value": complex(1, 2)}),
     (ds.floats, {"exclude_min": None}),
     (ds.floats, {"exclude_max": None}),
     (ds.floats, {"exclude_min": True}),  # because min_value=None
     (ds.floats, {"exclude_max": True}),  # because max_value=None
+    (ds.floats, {"min_value": 1.8, "width": 32}),
+    (ds.floats, {"max_value": 1.8, "width": 32}),
     (ds.fractions, {"min_value": 2, "max_value": 1}),
-    (ds.fractions, {"min_value": float("nan")}),
-    (ds.fractions, {"max_value": float("nan")}),
+    (ds.fractions, {"min_value": math.nan}),
+    (ds.fractions, {"max_value": math.nan}),
     (ds.fractions, {"max_denominator": 0}),
     (ds.fractions, {"max_denominator": 1.5}),
     (ds.fractions, {"min_value": complex(1, 2)}),
+    (ds.fractions, {"min_value": "1/3", "max_value": "1/2", "max_denominator": 2}),
+    (ds.fractions, {"min_value": "0", "max_value": "1/3", "max_denominator": 2}),
+    (ds.fractions, {"min_value": "1/3", "max_value": "1/3", "max_denominator": 2}),
     (ds.lists, {"elements": ds.integers(), "min_size": 10, "max_size": 9}),
     (ds.lists, {"elements": ds.integers(), "min_size": -10, "max_size": -9}),
     (ds.lists, {"elements": ds.integers(), "max_size": -9}),
     (ds.lists, {"elements": ds.integers(), "min_size": -10}),
-    (ds.lists, {"elements": ds.integers(), "min_size": float("nan")}),
+    (ds.lists, {"elements": ds.integers(), "min_size": math.nan}),
     (ds.lists, {"elements": ds.nothing(), "max_size": 1}),
     (ds.lists, {"elements": "hi"}),
     (ds.lists, {"elements": ds.integers(), "unique_by": 1}),
@@ -121,17 +126,19 @@ def fn_ktest(*fnkwargs):
     (ds.text, {"alphabet": [1]}),
     (ds.text, {"alphabet": ["abc"]}),
     (ds.binary, {"min_size": 10, "max_size": 9}),
-    (ds.floats, {"min_value": float("nan")}),
+    (ds.floats, {"min_value": math.nan}),
     (ds.floats, {"min_value": "0"}),
     (ds.floats, {"max_value": "0"}),
+    (ds.floats, {"min_value": 0.0, "max_value": -0.0}),
+    (ds.floats, {"min_value": 0.0, "max_value": 1.0, "allow_infinity": True}),
     (ds.floats, {"max_value": 0.0, "min_value": 1.0}),
     (ds.floats, {"min_value": 0.0, "allow_nan": True}),
     (ds.floats, {"max_value": 0.0, "allow_nan": True}),
     (ds.floats, {"min_value": 0.0, "max_value": 1.0, "allow_infinity": True}),
-    (ds.floats, {"min_value": float("inf"), "allow_infinity": False}),
-    (ds.floats, {"max_value": float("-inf"), "allow_infinity": False}),
-    (ds.complex_numbers, {"min_magnitude": float("nan")}),
-    (ds.complex_numbers, {"max_magnitude": float("nan")}),
+    (ds.floats, {"min_value": math.inf, "allow_infinity": False}),
+    (ds.floats, {"max_value": -math.inf, "allow_infinity": False}),
+    (ds.complex_numbers, {"min_magnitude": math.nan}),
+    (ds.complex_numbers, {"max_magnitude": math.nan}),
     (ds.complex_numbers, {"max_magnitude": complex(1, 2)}),
     (ds.complex_numbers, {"min_magnitude": -1}),
     (ds.complex_numbers, {"max_magnitude": -1}),
@@ -163,11 +170,18 @@ def fn_ktest(*fnkwargs):
     (ds.characters, {"max_codepoint": "1"}),
     (ds.characters, {"whitelist_categories": []}),
     (ds.characters, {"whitelist_categories": ["Nd"], "blacklist_categories": ["Nd"]}),
-    (ds.slices, {"size": 0}),
     (ds.slices, {"size": None}),
     (ds.slices, {"size": "chips"}),
     (ds.slices, {"size": -1}),
     (ds.slices, {"size": 2.3}),
+    (ds.sampled_from, {"elements": ()}),
+    (ds.ip_addresses, {"v": "4"}),
+    (ds.ip_addresses, {"v": 4.0}),
+    (ds.ip_addresses, {"v": 5}),
+    (ds.ip_addresses, {"v": 4, "network": "::/64"}),
+    (ds.ip_addresses, {"v": 6, "network": "127.0.0.0/8"}),
+    (ds.ip_addresses, {"network": b"127.0.0.0/8"}),  # only unicode strings are valid
+    (ds.ip_addresses, {"network": b"::/64"}),
 )
 def test_validates_keyword_arguments(fn, kwargs):
     with pytest.raises(InvalidArgument):
@@ -209,8 +223,8 @@ def test_validates_keyword_arguments(fn, kwargs):
     (ds.floats, {}),
     (ds.floats, {"min_value": 1.0}),
     (ds.floats, {"max_value": 1.0}),
-    (ds.floats, {"min_value": float("inf")}),
-    (ds.floats, {"max_value": float("-inf")}),
+    (ds.floats, {"min_value": math.inf}),
+    (ds.floats, {"max_value": -math.inf}),
     (ds.floats, {"max_value": 1.0, "min_value": -1.0}),
     (ds.floats, {"max_value": 1.0, "min_value": -1.0, "allow_infinity": False}),
     (ds.floats, {"min_value": 1.0, "allow_nan": False}),
@@ -225,7 +239,7 @@ def test_validates_keyword_arguments(fn, kwargs):
     (ds.complex_numbers, {"allow_nan": False}),
     (ds.complex_numbers, {"allow_nan": False, "allow_infinity": True}),
     (ds.complex_numbers, {"allow_nan": False, "allow_infinity": False}),
-    (ds.complex_numbers, {"max_magnitude": float("inf"), "allow_infinity": True}),
+    (ds.complex_numbers, {"max_magnitude": math.inf, "allow_infinity": True}),
     (ds.sampled_from, {"elements": [1]}),
     (ds.sampled_from, {"elements": [1, 2, 3]}),
     (ds.fixed_dictionaries, {"mapping": {1: ds.integers()}}),
@@ -236,12 +250,27 @@ def test_validates_keyword_arguments(fn, kwargs):
     (ds.text, {"alphabet": ds.sampled_from("abc")}),
     (ds.characters, {"whitelist_categories": ["N"]}),
     (ds.characters, {"blacklist_categories": []}),
+    (ds.ip_addresses, {}),
+    (ds.ip_addresses, {"v": 4}),
+    (ds.ip_addresses, {"v": 6}),
+    (ds.ip_addresses, {"network": "127.0.0.0/8"}),
+    (ds.ip_addresses, {"network": "::/64"}),
+    (ds.ip_addresses, {"v": 4, "network": "127.0.0.0/8"}),
+    (ds.ip_addresses, {"v": 6, "network": "::/64"}),
+    (ds.ip_addresses, {"network": IPv4Network("127.0.0.0/8")}),
+    (ds.ip_addresses, {"network": IPv6Network("::/64")}),
+    (ds.ip_addresses, {"v": 4, "network": IPv4Network("127.0.0.0/8")}),
+    (ds.ip_addresses, {"v": 6, "network": IPv6Network("::/64")}),
 )
 def test_produces_valid_examples_from_keyword(fn, kwargs):
     fn(**kwargs).example()
 
 
-@fn_test((ds.one_of, (1,)), (ds.tuples, (1,)))
+@fn_test(
+    (ds.one_of, (1,)),
+    (ds.one_of, (1, ds.integers())),
+    (ds.tuples, (1,)),
+)
 def test_validates_args(fn, args):
     with pytest.raises(InvalidArgument):
         fn(*args).example()
@@ -323,22 +352,21 @@ def test_fraction_lt_negative(x):
     assert x <= fractions.Fraction(-1, 2)
 
 
-@given(ds.decimals(min_value=-1.5, max_value=1.5, allow_nan=False))
+@given(ds.decimals(min_value=-1.5, max_value=1.5))
 def test_decimal_is_in_bounds(x):
-    # decimal.Decimal("-1.5") == -1.5 (not explicitly testable in py2.6)
     assert decimal.Decimal("-1.5") <= x <= decimal.Decimal("1.5")
 
 
 def test_float_can_find_max_value_inf():
-    assert minimal(ds.floats(max_value=float("inf")), lambda x: math.isinf(x)) == float(
+    assert minimal(ds.floats(max_value=math.inf), lambda x: math.isinf(x)) == float(
         "inf"
     )
-    assert minimal(ds.floats(min_value=0.0), lambda x: math.isinf(x)) == float("inf")
+    assert minimal(ds.floats(min_value=0.0), lambda x: math.isinf(x)) == math.inf
 
 
 def test_float_can_find_min_value_inf():
     minimal(ds.floats(), lambda x: x < 0 and math.isinf(x))
-    minimal(ds.floats(min_value=float("-inf"), max_value=0.0), lambda x: math.isinf(x))
+    minimal(ds.floats(min_value=-math.inf, max_value=0.0), lambda x: math.isinf(x))
 
 
 def test_can_find_none_list():
@@ -398,42 +426,6 @@ def test_minimal_iterable():
     assert list(minimal(ds.iterables(ds.integers()), lambda x: True)) == []
 
 
-@checks_deprecated_behaviour
-@pytest.mark.parametrize(
-    "fn,kwargs",
-    [
-        (ds.integers, {"min_value": decimal.Decimal("1.5")}),
-        (ds.integers, {"max_value": decimal.Decimal("1.5")}),
-        (ds.integers, {"min_value": -1.5, "max_value": -0.5}),
-        (ds.floats, {"min_value": 1.8, "width": 32}),
-        (ds.floats, {"max_value": 1.8, "width": 32}),
-        (ds.fractions, {"min_value": "1/3", "max_value": "1/2", "max_denominator": 2}),
-        (ds.fractions, {"min_value": "0", "max_value": "1/3", "max_denominator": 2}),
-    ],
-)
-def test_disallowed_bounds_are_deprecated(fn, kwargs):
-    fn(**kwargs).example()
-
-
-@checks_deprecated_behaviour
-@pytest.mark.parametrize(
-    "fn,kwargs", [(ds.integers, {"min_value": 0.1, "max_value": 0.2})]
-)
-def test_no_integers_in_bounds(fn, kwargs):
-    with pytest.raises(InvalidArgument):
-        fn(**kwargs).example()
-
-
-@checks_deprecated_behaviour
-@pytest.mark.parametrize(
-    "fn,kwargs",
-    [(ds.fractions, {"min_value": "1/3", "max_value": "1/3", "max_denominator": 2})],
-)
-def test_no_fractions_in_bounds(fn, kwargs):
-    with pytest.raises(InvalidArgument):
-        fn(**kwargs).example()
-
-
 @pytest.mark.parametrize("parameter_name", ["min_value", "max_value"])
 @pytest.mark.parametrize("value", [-1, 0, 1])
 def test_no_infinity_for_min_max_values(value, parameter_name):
@@ -458,7 +450,7 @@ def test_no_nan_for_min_max_values(value, parameter_name):
     test_not_nan()
 
 
-class Sneaky(object):
+class Sneaky:
     """It's like a strategy, but it's not a strategy."""
 
     is_empty = False
@@ -488,3 +480,17 @@ def test_chained_filter(x):
 def test_chained_filter_tracks_all_conditions():
     s = ds.integers().filter(bool).filter(lambda x: x % 3)
     assert len(s.flat_conditions) == 2
+
+
+@pytest.mark.parametrize("version", [4, 6])
+@given(data=ds.data())
+def test_ipaddress_from_network_is_always_correct_version(data, version):
+    ip = data.draw(ds.ip_addresses(v=version), label="address")
+    assert ip.version == version
+
+
+@given(data=ds.data(), network=ds.from_type(IPv4Network) | ds.from_type(IPv6Network))
+def test_ipaddress_from_network_is_always_in_network(data, network):
+    ip = data.draw(ds.ip_addresses(network=network), label="address")
+    assert ip in network
+    assert ip.version == network.version
